subroutine SPS1R(n,x,y,ind,ib,ax,bx,rho,zs,zm,xx,sp,dsp,d2sp)
!Построение сглаживающего кубического сплайна
!Исправленная программа "Smspline" из книги:
!Е.Шикин, А.Плис Кривые и поверхности на экране
!компьютера. М. "Диалог-МИФИ" 1996
integer, intent(in):: n, ind, ib
real, intent(in):: x(:), y(:), rho(:)
real, intent(in):: ax, bx, xx
real, intent(inout):: zs(:), zm(:)
real, intent(out):: sp, dsp, d2sp
real, allocatable:: a(:),b(:),c(:),d(:),e(:),g(:)
real h, h1, h2, h3, h4, di, tt
integer j, nj, nn, ierr
!begin
    if (n < 4) then
        zs(:)=0.0; zm(:)=0.0
        sp=0.0; dsp=0.0; d2sp=0.0
        return
    end if
!C
    if (ind==0) then
        allocate (a(n),b(n),c(n),d(n),e(n),g(n))
!C
        select case(ib)
        case(1)
            h1=x(2)-x(1); h2=x(3)-x(2)
            a(1)=h1/3.+(rho(1)+rho(2))/h1**2
            b(1)=h1/6.-(1./h1+1./h2)*rho(2)/h1-rho(1)/h1**2
            c(1)=rho(2)/(h1*h2)
            g(1)=(y(2)-y(1))/h1-ax
            e(3)=c(1)
            h1=x(n)-x(n-1); h2=x(n-1)-x(n-2)
            a(n)=h1/3.+(rho(n-1)+rho(n))/h1**2
            b(n-1)=h1/6.-(1./h1+1./h2)*rho(n-1)/h1-rho(n)/h1**2
            c(n-2)=rho(n-1)/(h1*h2)
            e(1)=0.; e(2)=0.; d(1)=0.; d(2)=b(1)
            c(n-1)=0.; c(n)=0.; b(n)=0.; d(n)=b(n-1)
            g(n)=bx-(y(n)-y(n-1))/h1; e(n)=c(n-2)
            nn=n
        case(2)
            a(1)=1.; a(n)=1.; b(1)=0.; b(n-1)=0.
            c(1)=0.; c(n-2)=0.; g(1)=0.; g(n)=0.
            e(1)=0.; e(2)=0.; e(3)=0.; d(1)=0.
            b(n)=0.; c(n)=0.; c(n-1)=0.
            nn=n
        case(3)
            h1=x(2)-x(1)
            h2=x(n)-x(n-1)
            h3=x(3)-x(2)
            a(1)=(h1+h2)/3.+rho(n-1)/h2**2+rho(2)/h1**2+  &
                (1./h1+1./h2)**2*rho(1)
            b(1)=h1/6.-((1./h2+1./h1)*rho(1)+(1./h2+1./h3)*rho(2))/h2
            c(1)=rho(2)/(h1*h3)
            g(1)=(y(2)-y(1))/h1-(y(n)-y(n-1))/h2
            d(2)=b(1); e(3)=c(1)
            h4=x(n-1)-x(n-2)
            c(n-2)=rho(n-1)/(h2*h4)
            c(n-1)=rho(n)/(h2*h1)
            b(n-1)=h2/3.-((1./h4+1./h2)*rho(n-1)+  &
                (1./h2+1./h1)*rho(n))/h2
            d(1)=b(n-1); e(1)=c(n-2); e(2)=c(n-1)
            g(n-1)=(y(n)-y(n-1))/h2-(y(n-1)-y(n-2))/h4
            nn=n-1
        end select
!C
        do j=2, n-1
            h1=x(j)-x(j-1); h2=x(j+1)-x(j)
            a(j)=(h1+h2)/3.+rho(j-1)/h1**2+rho(j+1)/h2**2+  &
                (1./h1+1./h2)**2*rho(j)
            g(j)=(y(j+1)-y(j))/h2-(y(j)-y(j-1))/h1
            if(j.le.n-2) then
                h3=x(j+2)-x(j+1)
                b(j)=h2/6.-((1./h1+1./h2)*rho(j)+  &
                    (1./h2+1./h3)*rho(j+1))/h2
                d(j+1)=b(j)
            end if
            if(j.le.n-3) then
                c(j)=rho(j+1)/(h2*h3)
                e(j+2)=c(j)
            end if
        end do
!C
        call LA31(a(1:nn),b(1:nn),c(1:nn),d(1:nn),e(1:nn),  &
			g(1:nn),zm(1:nn),ierr)
!C
        zs(1)=y(1)-rho(1)*(zm(2)-zm(1))/(x(2)-x(1))
        zs(n)=y(n)+rho(n)*(zm(n)-zm(n-1))/(x(n)-x(n-1))
        if (ib.eq.3) then
            zm(n)=zm(1)
            di=(zm(2)-zm(1))/(x(2)-x(1))
            di=di-(zm(n)-zm(n-1))/(x(n)-x(n-1))
            zs(1)=y(1)-rho(1)*di
            zs(n)=zs(1)
        end if
        do j=2, n-1
            di=(zm(j+1)-zm(j))/(x(j+1)-x(j))
            di=di-(zm(j)-zm(j-1))/(x(j)-x(j-1))
            zs(j)=y(j)-rho(j)*di
        end do
!C
        deallocate (a,b,c,d,e,g)
        sp=0.; dsp=0.; d2sp=0.
    else
!C
        do j=2, n
            nj=j
            if (x(j) > xx) exit
        end do
!C
        j=nj-1
        h=x(j+1)-x(j)
        tt=(xx-x(j))/h
        sp=zs(j)*(1.-tt)+zs(j+1)*tt-h**2/6.*tt*(1.-tt)*  &
            ((2.-tt)*zm(j)+(1.+tt)*zm(j+1))
        dsp=(zs(j+1)-zs(j))/h-h/6.*((2.-6.*tt+3.*tt**2)*zm(j)+  &
            (1.-3.*tt**2)*zm(j+1))
        d2sp=(1.-tt)*zm(j)+tt*zm(j+1)
    end if
!C
    return
end subroutine SPS1R